#!/bin/bash
# Copyright start
# Copyright (C) 2008 - 2022 Fortinet Inc.
# All rights reserved.
# FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
# Copyright end

set_logger() {
    local LOGDIR="/var/log/cyops/extension"
    mkdir -p $LOGDIR
    local LOGFILE="$LOGDIR/boot.log"
    exec > >(awk '{ print strftime("[%Y-%m-%d %H:%M:%S %Z]"), $0; fflush(); }' | tee --append $LOGFILE)
    exec 2>&1
}

init() {
    local fsr_docker_utils_sh="/opt/cyops/configs/scripts/docker/utils.sh"
    source $fsr_docker_utils_sh
}

create_docker_flag() {
    touch $fdc_flag_fsr_in_docker
    chmod 644 $fdc_flag_fsr_in_docker
}

check_if_pause_boot_config() {
    while [ -f "$fdc_flag_pause_boot_config" ]; do
        echo "Sleeping for 10 secs as flag [$fdc_flag_pause_boot_config] exist"
        sleep 10
    done
}

clean_up() {
    echo "Cleaning directory [$fdc_dir_backup]"
    rm -rf $fdc_dir_backup
}

is_license_installed() {
    local cmd_license_check="csadm license --show-details"
    local out=$($cmd_license_check)
    #
    # If license is not found, we get below msg:
    # License not found on the system.
    # We are checking for only 'not found on'
    #
    if echo $out | grep "not found on"; then
        # 1 means false
        return 1
    fi
    # 0 means true
    return 0
}

setup_cron_for_periodic_backup() {
    #
    # Set cron job to take backup of configuration after every 1 hour
    #
    echo "Setting up the cron job for configuration backup"
    echo "0 */1 * * * root sh $fdc_cron_config_handler" > /etc/cron.d/cron-config-handler
}

is_fresh_install() {
    if [ -f "$fdc_flag_vm_configured" ] || [ -f "$fdc_flag_vm_configuration_failed" ]; then
        # 1 means false
        return 1
    fi
    # 0 means true
    return 0
}

#
# Purpose of this sub-routine is to run the tasks for first boot on docker, be it extension or normal
# docker.
#
tasks_for_first_boot() {
    #
    # 1. Run the config wizard
    # 2. Update hostname and fqhn in database
    # 3. Configured the Embedded SME
    #

    #
    # Run the config VM. Config VM needs services like nginx, rabbitmq etc. to come up
    # before actual starting. Hence wait for 15 secs.
    #

    local sleep_sec=15
    echo "Sleeping for $sleep_sec secs before starting the Config VM"
    sleep $sleep_sec

    echo "Starting the Config VM"

    /opt/cyops/configs/scripts/config-vm.sh --non-interactive
    if [ -f "$fdc_flag_vm_configured" ]; then
        notify_ui_fortisoar_state  '{"steps": [{"label" : "Config VM", "success": true}]}'
    else
        PROVISIONING_SUCCEED="False"
        notify_ui_fortisoar_state  '{"steps": [{"label" : "Config VM", "success": false}]}'
    fi


    # Updates hostname and fqhn in database
    tasks_for_every_boot_on_extn

    #
    # Enable the embedded SME
    #
    # For docker, the TCP port will always be 55671. This is handled in codebase.
    # And password for GUI is changeme.
    #

    echo "Starting the Embedded SME enable"

    csadm secure-message-exchange enable --no-interaction
    if [ $? -eq 0 ]; then
        notify_ui_fortisoar_state  '{"steps": [{"label" : "Embedded SME Configuration", "success": true}]}'
    else
        PROVISIONING_SUCCEED="False"
        notify_ui_fortisoar_state  '{"steps": [{"label" : "Embedded SME Configuration", "success": false]}'
    fi
}

tasks_for_first_boot_on_extn() {
    if ! is_extension; then
        return
    fi

    echo "Installing the extension license"
    /opt/cyops-auth/utilities/licensemanager.py --deploy-extension-license
    if [ $? -eq 0 ]; then
        notify_ui_fortisoar_state  '{"steps": [{"label" : "Extension License Installation", "success": true}]}'
    else
        PROVISIONING_SUCCEED="False"
        notify_ui_fortisoar_state  '{"steps": [{"label" : "Extension License Installation", "success": false}]}'
    fi

    echo "Deactivating csadmin user"


    # csadmin is kept as super admin by default. Being super admin, it can not be marked inactive. We need to clear
    # super admin first before marking it inactive
    /opt/cyops/configs/scripts/api_caller.py \
           --endpoint "https://localhost/api/auth/config" \
           --method PUT --payload "{\"option\": \"super_admin\", \"value\":\"\"}"

    csadmin_das_uuid=$(psql -U cyberpgsql -d das -qAt -c "SELECT uuid FROM users WHERE login_id='csadmin'")
    csadmin_ch_uuid=$(psql -U cyberpgsql -d venom -qAt -c "SELECT uuid FROM actors WHERE firstname='CS' and lastname='Admin'")

    # Deactivate DAS part
    /opt/cyops/configs/scripts/api_caller.py \
           --endpoint "https://localhost/api/auth/users" \
           --method PUT --payload "{\"update\":{\"uuid\":\"$csadmin_das_uuid\",\"status\":2}}"

    # Deactivate CH part
    /opt/cyops/configs/scripts/api_caller.py \
        --endpoint "https://localhost/api/3/people/$csadmin_ch_uuid" \
        --method PUT --payload "{\"csActive\":false}"
}

tasks_for_every_boot_on_extn() {
    local hostname_extn_host="$GLOBAL_VAR_EXTN_HOSTNAME"
    if [ "X$hostname_extn_host" = "X" ]; then
        echo "ERROR: Failed to find the docker host's hostname"
        return
    fi

    local server_fqhn="${hostname_extn_host}/${fdc_extn_fsr_uri_start_keyword}"

    echo "Updating the server FQDN for workflow as [${server_fqhn}]"

    psql -U cyberpgsql sealab -c "update dynamic_variable_dynamicvariable set value = '$server_fqhn' where name='Server_fqhn'"

    local crudhub_global_hostname="${hostname_extn_host}/${fdc_extn_fsr_uri_start_keyword}"

    echo "Updating the hostname in system settings as [$crudhub_global_hostname]"

    /opt/cyops/configs/scripts/api_caller.py \
               --endpoint "https://localhost/api/3/system_settings/845c05cc-05b3-450e-9afb-df6b6e436321" \
               --method PUT --payload "{\"globalValues\": { \"hostname\": \"$crudhub_global_hostname\"}}" >/dev/null
}

perform_post_license_deployment_steps() {
    echo "Performing post license deployment steps"
    ln -sf /opt/cyops-routing-agent/postman/postman/postman_wsgi.ini /etc/uwsgi.d/postman_wsgi.ini
    systemctl enable cyops-postman.service cyops-integrations-agent.service
    systemctl restart cyops-postman.service cyops-integrations-agent.service
}

perform_config_wizard_steps() {
    echo "Enabling and starting the HA service"
    systemctl enable cyops-ha.service
    systemctl start cyops-ha.service
}

clean_up_last_restored_backup() {
    #
    # when backup is restored, it left the extracted directory. for eg.
    # [root@6d551f3d7288 .do-not-delete-backups]# ls -d */
    # 20201022_121047
    # clean those
    #

    local dirs_to_clean=$(ls -d $fdc_dir_backup/*/)
    for dir_to_clean in $dirs_to_clean
    do
        if [ -d $dir_to_clean ]; then
            rm -rf $dir_to_clean
        fi
    done
}

restore_backup() {
    local backup_file=$1
    local clean_backup_file=$2
    if [ "X$backup_file" = "X" ]; then
        backup_file=$(ls -trh $fdc_dir_backup/DR_BACKUP_*.tgz | tail -1)
    fi

    if [ "X$backup_file" = "X" ]; then
        return
    fi

    {
        echo "Restoring the backup [$backup_file]"
        csadm db --restore $backup_file
        echo "Restore completed [$backup_file]"
    } &> >(awk '{ print strftime("[%Y-%m-%d %H:%M:%S %Z]"), $0; fflush(); }' | tee --append /var/log/cyops/extension/restore.log)

    if [ "X$clean_backup_file" = "X1" ]; then
        local dir_to_delete=$(dirname $backup_file)
        echo "Deleting [$dir_to_delete]"
        rm -rf $dir_to_delete
    fi
}

handle_first_boot() {
    echo "Handling the first boot"
    clean_up
    tasks_for_first_boot
    if is_extension; then
        tasks_for_first_boot_on_extn
    fi
}

handle_subsequent_boot() {
    echo "Handling the subsequent boot"

    perform_config_wizard_steps

    if is_license_installed; then
        perform_post_license_deployment_steps
    fi

    clean_up_last_restored_backup

    restore_backup

    if is_extension; then
        tasks_for_every_boot_on_extn
    fi
}

my_exit() {
    local exit_status=$1
    mkdir -p $fdc_dir_backup
    chmod 775 $fdc_dir_backup
    # We need group as nginx as mea restore API written in DAS write the backup file
    # at this backup dir and remove the previous file.
    #
    chown root:nginx $fdc_dir_backup
    csadm db --backup-config $fdc_dir_backup
    setup_cron_for_periodic_backup
    echo "Exiting the boot-config.sh with exit code as [$exit_status]"
    exit $exit_status
}

##################
# Main/main/Main starts here
##############

set_logger

PROVISIONING_SUCCEED="True"

echo "Starting the boot-config.sh"

set +e

init

create_docker_flag

# Sometimes developer need a window to login to container and see the DB, config files etc.
# before the restore happens. For such case, developer need to touch the file on FMG and then
# start the FSR container.
# touch /var/docker/fortisoar/.pause-boot-config

check_if_pause_boot_config

# Docker image do not persist setfacl.
# https://github.com/moby/moby/issues/40553
# Hence, we are running below script, which configures required setfacl for running
# integrations using fsr-integrations user.
#
/opt/cyops-integrations/package/setup.sh

# When FSR docker runs in non extension environment, below script may throw some error,
# which is expected.
#

$fdc_extn_settings --create-secret
$fdc_extn_settings --set-timezone

if is_extension; then

    initialize_global_variables

    add_self_entry_in_etc_hosts

    if is_fresh_install; then
        state="{\"provision\": false, \"state\": \"init\", \"ip\": \"$GLOBAL_VAR_EXTN_IP\"}"
        notify_ui_fortisoar_state "$state"
    else
        state="{\"provision\": false, \"state\": \"restore\", \"ip\": \"$GLOBAL_VAR_EXTN_IP\"}"
        notify_ui_fortisoar_state "$state"
    fi

    sed -i "s#<base href=\"/\">#<base href=\"/$fdc_extn_fsr_uri_start_keyword/\">#g" /opt/cyops-ui/cindex.html
fi

if is_fresh_install; then
    handle_first_boot
else
    handle_subsequent_boot
fi

if is_extension; then
    if [ "X$PROVISIONING_SUCCEED" = "XTrue" ]; then
        notify_ui_fortisoar_state '{"provision": true, "state": "provisioned"}'
    else
        notify_ui_fortisoar_state '{"provision": true, "state": "failed"}'
    fi
fi

my_exit 0